#!/usr/bin/env python
# -*- coding: utf-8 -*-

# django-simple-webservice
# Copyright (c) 2014, Liricus, All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.


#==============================================================================
# DOCS
#==============================================================================

"""Django app for simple creation webservices json-rpc like

"""


#==============================================================================
# IMPORTS
#==============================================================================

from simple_webservice.core import webservice_autodiscover
from simple_webservice.core import register_model
from simple_webservice.core import user_by_session_id
from simple_webservice.core import parse_data
from simple_webservice.calls import register_call
from simple_webservice.serializer import query_to_dict, model_to_dict


#==============================================================================
# MAIN
#==============================================================================

if __name__ == "__main__":
    print(__doc__)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# django-simple-webservice
# Copyright (c) 2014, Liricus, All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.


#==============================================================================
# DOCS
#==============================================================================

"""This module contains the default calls shipped with simple-webservice and
some funtions for registering new ones

"""


#==============================================================================
# IMPORTS
#==============================================================================

import logging

from django.contrib.auth import authenticate, login as login_user
from django.contrib.sessions.models import Session

from simple_webservice import core, serializer


#==============================================================================
# LOG
#==============================================================================

# Get an instance of a logger
logger = logging.getLogger(__name__)


#==============================================================================
# REGISTER CALLS
#==============================================================================

_calls = {}


def register_call(login=False):
    """Decorator for register a new function as webservice

    :param login: If it's ``True`` the webservice the webservice require
                  authentification.
    :type login: bool

    Example
    -------

    ::

        @register_call
        def my_fancy_new_ws(*args, **kwargs):
            return {}


    """
    def _wrap(fnc):
        _calls[fnc.__name__] = {"login": login, "function": fnc}
        return fnc
    return _wrap


def execute(request, fncname, args, session_id):
    """Execute funcion (internal use

    """
    fnc_data = _calls[fncname]
    if fnc_data["login"]:
        core.user_by_session_id(session_id)
    return fnc_data["function"](request=request, session=session_id, **args)


#==============================================================================
# CALLS
#==============================================================================

@register_call()
def ping(**kwargs):
    """Always return True"""
    return {"ping": True}


@register_call()
def login(username, password, **kwargs):
    """Authentificate some user and return the session key

    """
    user = authenticate(username=username, password=password)
    request = kwargs.get("request")
    if user is not None:
        if user.is_active:
            login_user(request, user)
            session_id = request.session.session_key
            return {"session_id": session_id, "login": True}
    else:
        raise Exception("Invalid credentials")


@register_call(login=True)
def logout(session, **kwargs):
    """Delete session of logged in user

    """
    session = Session.objects.get(pk=session)
    session.delete()
    return {"login": False}


@register_call()
def check_session(session, **kwargs):
    """Verify if a session is still active"""
    try:
        Session.objects.get(pk=session)
        return {"alive": True}
    except Session.DoesNotExist:
        return {"alive": False}


@register_call()
@core.crud_function("insert")
def insert(Model, modelname, data, **kw):
    """Execute insert sql method in the given model"""
    mdl = Model.objects.create(**core.parse_data(data, Model))
    mdl.save()
    return {"pk": mdl.pk}


@register_call()
@core.crud_function("update")
def update(Model, modelname, pk, data, **kw):
    """Execute update query over given model for given primary key"""
    Model.objects.filter(pk=pk).update(**core.parse_data(data, Model))
    return {"pk": pk}


@register_call()
@core.crud_function("delete")
def delete(Model, modelname, pk, **kw):
    """Delete some object froma database with a given pk and the given model"""
    Model.objects.filter(pk=pk).delete()
    return {"pk": pk}


@register_call()
@core.crud_function("select")
def select(Model, modelname, data, **kw):
    """Excecute select statement over given model"""
    if data:
        query = Model.objects.filter(**core.parse_data(data, Model))
    else:
        query = Model.objects.all()
    return serializer.query_to_dict(query)


#==============================================================================
# MAIN
#==============================================================================

if __name__ == "__main__":
    print(__doc__)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# django-simple-webservice
# Copyright (c) 2014, Liricus, All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.


#==============================================================================
# DOCS
#==============================================================================

"""Django views

"""


#==============================================================================
# IMPORTS
#==============================================================================

import json
import traceback
import sys
import logging

from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.conf import settings
from django.db import transaction

from simple_webservice import calls


#==============================================================================
# LOG
#==============================================================================

log = logging.getLogger('simplewebservice')


#==============================================================================
# VIEWS
#==============================================================================

@csrf_exempt
@require_POST
@transaction.atomic
def call(request):
    """
    query = {
        "id": arbitrario
        "name": nombre de funcion a ejecutar
        "args": argumen tos de la funcion
        "session": token de la funcion
    }

    return = {
        "id": igual que el id que fue
        "error": bool
        "error_msg": si error es true
        "stacktrace": string
        "response": puede ser none o el resultado
    }

    """
    qid, qname, args = None, None, None
    error, response, error_msg, stacktrace = False, None, "", ""
    try:
        data = None
        if "query" in request.POST:
            data = json.loads(request.POST["query"])
        else:
            data = json.loads(request.body)
        qid = data["id"]
        qname = data["name"]
        qargs = data["args"]
        session_id = data.get("session")
        response = calls.execute(request, qname, qargs, session_id)
    except Exception as err:
        error = True
        error_msg = unicode(err)
        if settings.DEBUG:
            stacktrace = u"".join(
                traceback.format_exception(*sys.exc_info())
            )
            print stacktrace

    data = {"id": qid, "error": error, "error_msg": error_msg,
            "stacktrace": stacktrace, "response": response}
    sdata = json.dumps(data)
    log.debug(sdata)
    return HttpResponse(sdata, content_type='application/json')


#==============================================================================
# MAIN
#==============================================================================

if __name__ == "__main__":
    print(__doc__)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# django-simple-webservice
# Copyright (c) 2014, Liricus, All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.


#==============================================================================
# DOCS
#==============================================================================

"""Core functionalities of dsws

"""


#==============================================================================
# IMPORTS
#==============================================================================

import logging
import datetime
import functools

from django.contrib.auth.models import User
from django.contrib.sessions.models import Session
from django.utils.importlib import import_module
from django.db import models


#==============================================================================
# LOG
#==============================================================================

log = logging.getLogger('simplewebservice')


#==============================================================================
# CONF
#==============================================================================

LOADING_SIMPLE_WEBSERVICE = False

REGISTERED_MODELS = {"select": {}, "insert": {}, "delete": {}, "update": {}}

PARSERS = {
    models.DateTimeField: lambda x, f: (
        datetime.datetime.strptime(x, "%Y-%m-%d %H:%M:%S.%f")
        if isinstance(x, basestring) else x
    ),
    models.TimeField: lambda x, f: (
        datetime.datetime.strptime(x, "%H:%M:%S.%f").time()
        if isinstance(x, basestring) else x
    ),
    models.DateField: lambda x, f: (
        datetime.datetime.strptime(x, "%Y-%m-%d").date()
        if isinstance(x, basestring) else x
    ),
    models.ForeignKey: lambda x, f: f.related_field.model.objects.get(pk=x)
}


DEFAULT_PARSER = lambda x, f: x


#==============================================================================
# FUNCTIONS
#==============================================================================

def webservice_autodiscover():
    """
    Auto-discover INSTALLED_APPS webservices.py modules and fail silently when
    not present. NOTE: autodiscover was inspired/copied from
    django.contrib.admin autodiscover

    """
    global LOADING_SIMPLE_WEBSERVICE
    if LOADING_SIMPLE_WEBSERVICE:
        return
    LOADING_SIMPLE_WEBSERVICE = True

    import imp
    from django.conf import settings

    for app in settings.INSTALLED_APPS:

        try:
            app_path = import_module(app).__path__
        except AttributeError:
            continue

        try:
            imp.find_module('webservices', app_path)
        except ImportError:
            continue

        import_module("%s.webservices" % app)

    LOADING_SIMPLE_WEBSERVICE = False


def user_by_session_id(session_id):
    """Utility for recover an User from their session id"""
    session = Session.objects.get(pk=session_id)
    return User.objects.get(pk=session.get_decoded()['_auth_user_id'])


def register_model(Model, select=False, insert=False,
                   update=False, delete=False, login=False):
    """Register a model for access for traditional queries
    (insert/select/update/delete)

    You can configure your model for allow execute the the query with
    authentification

    For example, if you want to allow a webservice for execute a select method
    as annonymous but the insert and delete as registered
    user with the command:

    ::

        register.model(auth.models.User, select=True)
        register.model(auth.models.User, insert=True, delete=True, login=True)

    """
    name = "{}.{}".format(Model._meta.app_label, Model.__name__)
    if select:
        REGISTERED_MODELS["select"][name] = Model, login
    if insert:
        REGISTERED_MODELS["insert"][name] = Model, login
    if delete:
        REGISTERED_MODELS["delete"][name] = Model, login
    if update:
        REGISTERED_MODELS["update"][name] = Model, login


def model_by_op(model_name, op):
    """Retrieve model by query name"""
    try:
        return REGISTERED_MODELS[op][model_name]
    except:
        raise ValueError(
            "Model '{}' do not allowed for {}".format(model_name, op)
        )


def parse_data(data, mdl):
    """Convert raw json query into model like objects"""
    fields = dict((f.name, f) for f in mdl._meta.fields)
    parsed = {}
    for k, v in data.items():
        field = fields.get(k)
        if not field:
            field = fields[k.rsplit("__", 1)[0]]
        parsed[k] = PARSERS.get(type(field), DEFAULT_PARSER)(v, field)
    return parsed


def crud_function(op):
    """Wraps functionas crus one"""
    def _inner(fnc):
        @functools.wraps(fnc)
        def _wrap(modelname, *a, **kw):
            Model, login = model_by_op(modelname, op)
            if login:
                user_by_session_id(kw["session"])
            return fnc(Model, modelname, *a, **kw)
        return _wrap
    return _inner


#==============================================================================
# MAIN
#==============================================================================

if __name__ == "__main__":
    print(__doc__)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# django-simple-webservice
# Copyright (c) 2014, Liricus, All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.


#==============================================================================
# DOCS
#==============================================================================

"""This file is for distribute

"""


#==============================================================================
# IMPORTS
#==============================================================================

import os

from ez_setup import use_setuptools
use_setuptools()

from setuptools import setup, find_packages

#==============================================================================
# CONSTANTS
#==============================================================================

PATH = os.path.abspath(os.path.dirname(__file__))

REQUIREMENTS = [
    "Django>=1.6.5"
]


#==============================================================================
# META
#==============================================================================

# : This is the project name
PRJ = "django-simple-webservice"

# : The project version as tuple of strings
VERSION = ("0", "1")

# : The project version as string
STR_VERSION = ".".join(VERSION)
__version__ = STR_VERSION

# : For "what" is usefull simple-ws
DOC = """Django app for simple create  json-rpc like webservices"""

# : The short description for pypi
SHORT_DESCRIPTION = []
for line in DOC.splitlines():
    if not line.strip():
        break
    SHORT_DESCRIPTION.append(line)
SHORT_DESCRIPTION = u" ".join(SHORT_DESCRIPTION)
del line

# : Clasifiers for optimize search in pypi
CLASSIFIERS = (
    "Development Status :: 4 - Beta",
    "Environment :: Web Environment",
    "Framework :: Django",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: BSD License",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.7",
    "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
    "Topic :: Software Development :: Libraries :: Application Frameworks",
)

# : Home Page of simple-ws
URL = "https://bitbucket.org/liricus/django-simple-webservice"

# : Author of this simple-ws
AUTHOR = "Liricus SRL"

# : Email ot the autor
EMAIL = "info@liricus.com.ar"

# : The license name
LICENSE = "LGPL"

# : The license of simple-ws
FULL_LICENSE = u"""" django-simple-webservice
Copyright (c) 2014, Liricus, All rights reserved.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 3.0 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.

"""

# : Keywords for search of pypi
KEYWORDS = """webservices json simple django"""


#==============================================================================
# FUNCTIONS
#==============================================================================

setup(
    name=PRJ.lower(),
    version=STR_VERSION,
    description=SHORT_DESCRIPTION,
    author=AUTHOR,
    author_email=EMAIL,
    url=URL,
    license=LICENSE,
    keywords=KEYWORDS,
    classifiers=CLASSIFIERS,
    packages=[pkg for pkg in find_packages() if pkg.startswith("simple_webservice")],
    include_package_data=True,
    py_modules=["ez_setup"],
    install_requires=REQUIREMENTS,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# django-simple-webservice
# Copyright (c) 2014, Liricus, All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3.0 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library.


#==============================================================================
# DOCS
#==============================================================================

"""Serializers

"""


#==============================================================================
# IMPORTS
#==============================================================================

import datetime
import decimal

from django.forms.models import model_to_dict as _model_to_dict


#==============================================================================
# CONSTANTS
#==============================================================================

TO_SIMPLE_TYPES = {
    datetime.datetime: lambda x: x.isoformat().replace("T", " "),
    datetime.time: lambda x: x.isoformat(),
    datetime.date: lambda x: x.isoformat(),
    bool: lambda x: x,
    int: lambda x: x,
    long: lambda x: x,
    float: lambda x: x,
    str: unicode,
    unicode: lambda x: x,
    decimal.Decimal: lambda x: unicode(x),
    type(None): lambda x: None,
    complex: lambda x: unicode(x)
}


DEFAULT_PARSER = lambda x: unicode(x)


#==============================================================================
# FUNCTIONS
#==============================================================================

def query_to_dict(query, *a, **kw):
    """Convert a django query ito a list of json-serialisable objects"""
    return [model_to_dict(obj) for obj in query]


def model_to_dict(obj, *a, **kw):
    """Convert a single model into json serialisable object"""
    as_dict = _model_to_dict(obj, *a, **kw)
    for k, v in as_dict.items():
        as_dict[k] = TO_SIMPLE_TYPES.get(type(v), DEFAULT_PARSER)(v)
    return as_dict


#==============================================================================
# MAIN
#==============================================================================

if __name__ == "__main__":
    print(__doc__)
